//------------file: T1.java-------------------------------------------------

import java.lang.*;

//A package is a collection of related classes and interfaces
//providing access protection and namespace management

//display listing of all source files: cat P1/*.java *.java | more
//save listing of all source files:    cat P1/*.java *.java > X1.txt

//application test file:       ./T1.java
//package files: ./P1/I1.java ./P1/I2.java ./P1/I3.java ./P1/A1.java ./P1/A2.java ./P1/C1.java ./P1/C2.java ./P1/C3.java
//package compile: javac -d ./P1 ./P1/I*.java ./P1/A*.java ./P1/C*.java
//application compile:           javac -classpath P1 T1.java
//application execute:           java -g T1
//application debug:             jdb T1
//application webpage document:  javadoc *
//application archive:           jar cvmf Manifest.mf eecs382apiP1.jar *
//application table of contents: jar tvf eecs382apiP1.jar
//application unarchive:         jar xvf eecs382apiP1.jar
//application execute jar:       java -jar eecs382apiP1.jar

import P1.*;            //package location determined by "-classpath" in javac command

public class T1 {
   public static void main(String [] argv) {
   //C1 c1;              //ok: implied package P1 and NOT instantiated; "only" an object reference (i.e. pointer)
   //C1 c1 = null;       //ok: implied package P1 and NOT instantiated but has a null pointer reference for checking in "if/then/else" statements;
   //C1 c1 = new C1();   //ok: implied package P1 and instantiated object reference c1 by using "new"
  P1.C1 c1 = new C1();   //ok: explicit package P1 reference to class C1 and instantiated;

     C1 c12;             //ok: not instantiated; only an object reference (i.e. pointer)
     C2 c2 = new C2();   //implied package P1 
     C3 c3 = new C3(); 

     A1 a1;
   //A1 a1=null;        //ok: null pointer reference for checking in "if/then/else" statements;
   //A1 a1=new A1();    //illegal cannot instantiate abstract class;
  P1.A2 a2;             //explicit package P1 reference for an "abstract" class

     I1 i1;
   //I1 i1=null;        //ok: null pointer reference for checking in "if/then/else" statements;
   //I1 i1=new I1();    //illegal cannot instantiate interface class: I1 i1=new I1();
     I2 i2;
  P1.I3 i3;             //explicit package P1 reference for an "interface"

     c1.m1();     //class C1 contains only method m1();
     c2.m2();     //class C2 contains only method m2();
     c2.m1();     //where did it get m1(); from C2? it inherited it from C1!
     c3.m1();     //class C3 is derived from class C1 and contains methods m1() and m2();
                  //Note: class C3 overrides m1() of class C1
     c3.m2(); 

     c12=c2;   //subclass instance c2 has m1() and m2(); it can be assigned to a
               //superclass instance c12 which only has m1() in it

     c12.m1(); //instance c12 of class C1 has only one method m1()
   //c12.m2(); //cannot do: although m2() is in class c2; c12 does not!
     c12=c3;
     c12.m1();
     System.out.println("--------");
     a1=c1;    //allowed: instance c1 of class C1 is derived from abstract class A1
     a1.m1();
     a1=c2;    //allowed: instance c2 is derived of c1 which is derived of a1
     a1.m1();

     i1=c1;    //allowed: because class C1 implements interface I1
     i1.m1();  
   //i2=c1;    //illegal: because class C1 does NOT implement I2!
   //i3=c1;    //illegal: because class C1 does NOT implement I3!

   //i1=c2;    //illegal: because class C2 does NOT implement I1!
     i2=c2;    //allowed: because class C2 implements interface I2
     i2.m1();  //interface I1 has contains two methods: m1() and m2()
     i2.m2();  //class C2 does not have m1() method. So why does this work?
               //It works because class C2 inherits the m1() from class C1

     i2=c3;
     i2.m1();
     i2.m2();
     i3=c2;
     i3.m1();
     i3.m2();
   //i3=c3;    //illegal: C3 doesn't implement interface I3;

     //c2 extends a1 not a2: can't do: a2=c1; or a2=c2; 
   }
}
